@@ -11,14 +11,19 @@ an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express o
 specific language governing permissions and limitations under the License.
 """
 
+import time
+import logging
 from typing import Optional
 
 from celery import current_app
+from bamboo_engine.eri.models import ExecuteInterruptPoint, ScheduleInterruptPoint
 
 from pipeline.eri.celery.queues import QueueResolver
 
 from pipeline.eri.models import Process
 
+logger = logging.getLogger("bamboo_engine")
+
 
 def _retry_once(action: callable):
     try:
@@ -31,17 +36,35 @@ def _retry_once(action: callable):
 
 
 class TaskMixin:
-    def _get_task_route_params(self, task_name: str, process_id: int):
-        process = Process.objects.filter(id=process_id).only("priority", "queue").first()
-        resolver = QueueResolver(process.queue)
+    def _get_task_route_params(self, task_name: str, queue: str, priority: int):
+        resolver = QueueResolver(queue)
         queue, routing_key = resolver.resolve_task_queue_and_routing_key(task_name)
         return {
             "queue": queue,
-            "priority": process.priority,
+            "priority": priority,
             "routing_key": routing_key,
         }
 
-    def execute(self, process_id: int, node_id: str, root_pipeline_id: str, parent_pipeline_id: str):
+    def _standardize_headers(self, headers: Optional[dict], process_id: int):
+        if headers is None:
+            headers = {}
+
+        if "route_info" not in headers:
+            process = Process.objects.filter(id=process_id).only("priority", "queue").first()
+            headers["route_info"] = {"queue": process.queue, "priority": process.priority}
+
+        headers["timestamp"] = time.time()
+        return headers
+
+    def execute(
+        self,
+        process_id: int,
+        node_id: str,
+        root_pipeline_id: str,
+        parent_pipeline_id: str,
+        recover_point: Optional[ExecuteInterruptPoint] = None,
+        headers: Optional[dict] = None,
+    ):
         """
         派发执行任务，执行任务被拉起执行时应该调用 Engine 实例的 execute 方法
 
@@ -51,18 +74,29 @@ class TaskMixin:
         :type node_id: str
         """
         task_name = "pipeline.eri.celery.tasks.execute"
-        route_params = self._get_task_route_params(task_name, process_id)
+        headers = self._standardize_headers(headers=headers, process_id=process_id)
+        route_params = self._get_task_route_params(
+            task_name=task_name, queue=headers["route_info"]["queue"], priority=headers["route_info"]["priority"]
+        )
 
         def action():
-            current_app.tasks[task_name].apply_async(
+            result = current_app.tasks[task_name].apply_async(
                 kwargs={
                     "process_id": process_id,
                     "node_id": node_id,
                     "root_pipeline_id": root_pipeline_id,
                     "parent_pipeline_id": parent_pipeline_id,
+                    "recover_point": "{}" if not recover_point else recover_point.to_json(),
+                    "headers": headers,
                 },
                 **route_params,
             )
+            logger.info(
+                "[pipeline-trace](root_pipeline: %s) node(%s) execute task %s sended",
+                root_pipeline_id,
+                node_id,
+                result.id,
+            )
 
         _retry_once(action=action)
 
@@ -72,6 +106,8 @@ class TaskMixin:
         node_id: str,
         schedule_id: str,
         callback_data_id: Optional[int] = None,
+        recover_point: Optional[ScheduleInterruptPoint] = None,
+        headers: Optional[dict] = None,
     ):
         """
         派发调度任务，调度任务被拉起执行时应该调用 Engine 实例的 schedule 方法
@@ -84,18 +120,24 @@ class TaskMixin:
         :type schedule_id: str
         """
         task_name = "pipeline.eri.celery.tasks.schedule"
-        route_params = self._get_task_route_params(task_name, process_id)
+        headers = self._standardize_headers(headers=headers, process_id=process_id)
+        route_params = self._get_task_route_params(
+            task_name=task_name, queue=headers["route_info"]["queue"], priority=headers["route_info"]["priority"]
+        )
 
         def action():
-            current_app.tasks[task_name].apply_async(
+            result = current_app.tasks[task_name].apply_async(
                 kwargs={
                     "process_id": process_id,
                     "node_id": node_id,
                     "schedule_id": schedule_id,
                     "callback_data_id": callback_data_id,
+                    "recover_point": "{}" if not recover_point else recover_point.to_json(),
+                    "headers": headers,
                 },
                 **route_params,
             )
+            logger.info("[pipeline-trace] node(%s) schedule task %s sended", node_id, result.id)
 
         _retry_once(action=action)
 
@@ -105,6 +147,7 @@ class TaskMixin:
         node_id: str,
         schedule_id: str,
         schedule_after: int,
+        headers: Optional[dict] = None,
         callback_data_id: Optional[int] = None,
     ):
         """
@@ -120,56 +163,25 @@ class TaskMixin:
         :type schedule_after: int
         """
         task_name = "pipeline.eri.celery.tasks.schedule"
-        route_params = self._get_task_route_params(task_name, process_id)
+        headers = self._standardize_headers(headers=headers, process_id=process_id)
+        route_params = self._get_task_route_params(
+            task_name=task_name, queue=headers["route_info"]["queue"], priority=headers["route_info"]["priority"]
+        )
+        headers["timestamp"] += schedule_after
 
         def action():
-            current_app.tasks[task_name].apply_async(
+            result = current_app.tasks[task_name].apply_async(
                 kwargs={
                     "process_id": process_id,
                     "node_id": node_id,
                     "schedule_id": schedule_id,
                     "callback_data_id": callback_data_id,
+                    "recover_point": "{}",
+                    "headers": headers,
                 },
                 countdown=schedule_after,
                 **route_params,
             )
+            logger.info("[pipeline-trace] node(%s) schedule task %s sended", node_id, result.id)
 
         _retry_once(action=action)
-
-    def start_timeout_monitor(self, process_id: int, node_id: str, version: str, timeout: int):
-        """
-        开始对某个节点执行的超时监控，若超时时间归零后节点未进入归档状态，则强制失败该节点
-
-        :param process_id: 进程 ID
-        :type process_id: int
-        :param node_id: 节点 ID
-        :type node_id: str
-        :param version: 执行版本
-        :type version: str
-        :param timeout: 超时时间，单位为秒
-        :type timeout: int
-        """
-        task_name = "pipeline.eri.celery.tasks.timeout_check"
-        route_params = self._get_task_route_params(task_name, process_id)
-
-        current_app.tasks[task_name].apply_async(
-            kwargs={"process_id": process_id, "node_id": node_id, "version": version}, countdown=timeout, **route_params
-        )
-
-    def stop_timeout_monitor(
-        self,
-        process_id: int,
-        node_id: str,
-        version: str,
-    ):
-        """
-        停止对某个节点的超时监控
-
-        :param process_id: 进程 ID
-        :type process_id: int
-        :param node_id: 节点 ID
-        :type node_id: str
-        :param version: 执行版本
-        :type version: str
-        """
-        return
